import os
import re
import time
import random

from multiprocessing import Pool, TimeoutError
#for FILE in *; do cp $FILE ../$(expr $(echo $FILE | cut -c 8) \* $(echo $FILE | cut -c 16) \* 7)_$FILE; done

command_minizinc = 'minizinc.exe'
bin_size = 25
max_sols = 1000
max_time = 300
base_dir = "mzn_output"

def prepare_dzn(name, nurse, week, l, u, k, check_doable):
	dzn_datas = (
		f"""l = {l};
		u = {u};
		k = {k};
		n_nurses = {nurse};
		n_weeks = {week};		
		""")
	dzn = f"{__create_partial_output__(name, l, u, k)}.dzn"
	seq = f"seq = seq_search([{'int_search(schedule, dom_w_deg, indomain_random),' if not check_doable else ''}restart_luby(10)]);"
	with open(dzn, 'w') as f:
		f.writelines([dzn_datas, seq])
		time.sleep(1)
	return dzn

def create_output(name, l, u, k, sols = max_sols):
	return __create_partial_output__(name, l, u, k) + f"{sols}_sols.json"

def __create_partial_output__(name, l, u, k):
	return f"{base_dir}/{name}_l_{l}_u_{u}_k_{k}_"

def create_cmd(mzn, dzn, name, check_doable):
	cmd = f"{command_minizinc} {mzn} {dzn} --solver {'Chuffed' if check_doable else 'Gecode'} -O5 "
	if not check_doable:
		cmd += f"-n {bin_size}"
	return cmd

def execute(nurse, week, mzn, name, l, u, k):
	partial_output = __create_partial_output__(name, l, u, k)
	sols = []	
	intermediate_sols = []
	dzn = prepare_dzn(name, nurse, week, l, u, k, True)
	cmd = create_cmd(mzn, dzn, partial_output, True)
	unsat = (len(execute_cmd(0, cmd, partial_output + ".json")) == 0)
	if not unsat:
		prepare_dzn(name, nurse, week,  l, u, k, False)
		cmd = create_cmd(mzn, dzn, partial_output, False)
		for it in range(max_sols // bin_size):
			intermediate_sols = execute_cmd(it+1, cmd, partial_output + ".json")
			sols = sols + intermediate_sols
	sols = list(set(sols))
	print(f"{len(sols)} solutions found")
	output = create_output(name, l, u, k, len(sols))
	with open(output, 'w') as f:
		f.writelines(sols)
	os.remove(dzn)

def execute_cmd(it, cmd, output):
	result = "UNKNOWN"
	restarts = 0
	while "UNKNOWN" in result:
		random_cmd =  cmd + f" -r {random.randint(0, 1E8)} -t {max_time*1000*max((restarts+1)*0.5, 1)} >> {output}"
		print(f"Doing {random_cmd}")
		print(f"Doing {it + restarts}/{(max_sols // bin_size) + restarts}", end='\r')
		_ = os.popen(random_cmd).read() 
		restarts += 1
		if os.path.isfile(output):
			result = open(output, 'r').read()
	if os.path.isfile(output):
		intermediate_sols =  open(output, 'r').read()
		intermediate_sols = intermediate_sols.replace('],\n[', ', ')
		intermediate_sols = intermediate_sols.replace('[[', '[')
		intermediate_sols = intermediate_sols.replace(']]', ']')
		intermediate_sols = intermediate_sols.split('----------\n')[:-1]
		os.remove(output)
	else:
		intermediate_sols = []
	return intermediate_sols

def main():
	n_nurses = range(2, 8)
	n_weeks = range(1, 6)

	to_execute = []

	files_dir = f'\n{base_dir}/' + f'\n{base_dir}/'.join(os.listdir(base_dir))

	for nurse in n_nurses:
		for week in n_weeks:
			if nurse * week * 7 > 60:
				continue
			mzns = ["Infirmieres.mzn", "Infirmieres_Complexe.mzn"]
			names = [f"nurses_{nurse}_weeks_{week}_sequence", f"nurses_{nurse}_weeks_{week}_complex"]
			for mzn, name in zip(mzns, names):
				begin = time.time()
				for l in range(1, 5):
					for u in range(2,8):
						for k in range(2,8):
							if l <= u and u < k:
								partial_output = __create_partial_output__(name, l, u, k)
								if re.search(f"{partial_output}", files_dir):
									continue
								to_execute.append((execute, (nurse, week, mzn, name, l, u, k)))
				print(f"Finished {name} in {time.time()-begin}")

	with Pool(processes=4) as pool:
		multiple_results = [pool.apply_async(f, args) for f, args in to_execute]
		try:
			_ = [res.get() for res in multiple_results]
		except:
			pass

if __name__ == '__main__':
	main()